# This function generates C code with NN model
# Copyright: T. Bajraszewski
# email: tomasz@bajraszewski.pl
# 2022
function generate_c_code(filename, varName, params, dataType)

  len = length(params);

  numParametersPerLine = 10;
  nLines  = fix(len/numParametersPerLine);
  nLastParams = len - nLines*numParametersPerLine;
  if nLastParams>0
    nLines = nLines + 1;
  endif

  fid = fopen(filename, 'a');

  #prolog
  fprintf(fid, "const %s %s[] PROGMEM = {\n", dataType, varName);
  #lines loop

  for i=1:nLines
    #parameters loop
    n = 0;
    if i==nLines && nLastParams>0
      n = nLastParams;
    else
      n = numParametersPerLine;
    endif
    for p=1:n
      if strcmp(dataType, 'float') || strcmp(dataType, 'double')
        fprintf(fid, "%.08f", params((i-1)*numParametersPerLine + p));
      elseif strcmp(dataType, 'int16')
        fprintf(fid, "%d", cast(params((i-1)*numParametersPerLine + p), "int16"));
      elseif strcmp(dataType, 'uint16')
        fprintf(fid, "%d", cast(params((i-1)*numParametersPerLine + p), "uint16"));
      elseif strcmp(dataType, 'int8')
        fprintf(fid, "%d", cast(params((i-1)*numParametersPerLine + p), "int8"));
      elseif strcmp(dataType, 'uint8')
        fprintf(fid, "%d", cast(params((i-1)*numParametersPerLine + p), "uint8"));
      endif
      if i==nLines && p==n
      else
        fprintf(fid, ",\t");
      endif
    endfor
   fprintf(fid, "\n");
  endfor

  #epilog
  fprintf(fid, "};\n\n");

  fclose(fid);

endfunction
