function [J grad] = nnCostFunction(nn_params, ...
                                   input_layer_size, ...
                                   hidden_layer_size, ...
                                   num_labels, ...
                                   X, y, lambda)
%NNCOSTFUNCTION Implements the neural network cost function for a two layer
%neural network which performs classification
%   [J grad] = NNCOSTFUNCTON(nn_params, hidden_layer_size, num_labels, ...
%   X, y, lambda) computes the cost and gradient of the neural network. The
%   parameters for the neural network are "unrolled" into the vector
%   nn_params and need to be converted back into the weight matrices. 
% 
%   The returned parameter grad should be a "unrolled" vector of the
%   partial derivatives of the neural network.
%

% Reshape nn_params back into the parameters Theta1 and Theta2, the weight matrices
% for our 2 layer neural network
#fprintf("Enter nnCostFunction\n");

Theta1 = reshape(nn_params(1:hidden_layer_size * (input_layer_size + 1)), ...
                 hidden_layer_size, (input_layer_size + 1));

Theta2 = reshape(nn_params((1 + (hidden_layer_size * (input_layer_size + 1))):end), ...
                 num_labels, (hidden_layer_size + 1));

% Setup some useful variables
m = size(X, 1);
#fprintf("Number of samples: ");
#disp(m)
% You need to return the following variables correctly 
J = 0;
Theta1_grad = zeros(size(Theta1));
Theta2_grad = zeros(size(Theta2));

% ====================== YOUR CODE HERE ======================
% Instructions: You should complete the code by working through the
%               following parts.
%
% Part 1: Feedforward the neural network and return the cost in the
%         variable J. After implementing Part 1, you can verify that your
%         cost function computation is correct by verifying the cost
%         computed in ex4.m
%
% Part 2: Implement the backpropagation algorithm to compute the gradients
%         Theta1_grad and Theta2_grad. You should return the partial derivatives of
%         the cost function with respect to Theta1 and Theta2 in Theta1_grad and
%         Theta2_grad, respectively. After implementing Part 2, you can check
%         that your implementation is correct by running checkNNGradients
%
%         Note: The vector y passed into the function is a vector of labels
%               containing values from 1..K. You need to map this vector into a 
%               binary vector of 1's and 0's to be used with the neural network
%               cost function.
%
%         Hint: We recommend implementing backpropagation using a for-loop
%               over the training examples if you are implementing it for the 
%               first time.
%
% Part 3: Implement regularization with the cost function and gradients.
%
%         Hint: You can implement this around the code for
%               backpropagation. That is, you can compute the gradients for
%               the regularization separately and then add them to Theta1_grad
%               and Theta2_grad from Part 2.
%

# Add bias to X
X = [ones(m, 1) X];

% X has size m x 401
% Theta1 has size 25 x 401
% Theta2 has size 10 x 26

# compute cost
for i=1:m
  # X(i,:) has size 1 x 401
  a2 = sigmoid(Theta1 * X(i,:)' );
  #disp(a2);
  #pause;
  # a2 is vector 25 x 1
  # add bias to a2
  a2 = [1; a2];
  #disp(a2);
  #pause;
  #compute a^(3) = h
  h = sigmoid(Theta2 * a2);
  
  # h has now size 10 x 1
  
  yv = zeros(1, num_labels);
  yv(1, y(i)) = 1;
  # y has size 1 x 10
  
  J = J + sum( yv*log(h) + (1-yv)*log(1-h) );
  
endfor

J = -J / m;

# adding regularization component

reg = sum(sum(Theta1(:,2:end) .^ 2));
reg = reg + sum(sum(Theta2(:,2:end) .^ 2));
J = J + reg * lambda / (2*m);

# Computing gradients

# for testing nn size which is 
  #input_layer_size X = 3;
  #hidden_layer_size = 5;
  #num_labels = 3;
# Theta1 has size 5 x 4
# Theta2 has size 3 x 6
# so total is 20+18 = 38 thetas (nn_params)

#fprintf("Computing gradients\n");

DD1 = zeros(size(Theta1));
DD2 = zeros(size(Theta2));
for t=1:m
#  fprintf("  i=%d\n", t);
  # step1: compute forward prop.
  a1 = X(t,:)'; #a1 size 1x4
  z2 = Theta1 * a1; #size 5x4 * 4x1 = 5x1
  
  a2 = sigmoid(z2); #size 5x1
  a2 = [1; a2]; #size 6x1
  z3 = Theta2 * a2;  #size 3x6 * 6x1 = 3x1
  a3 = sigmoid(z3);  #3x1

  yv = zeros(1, num_labels);
  yv(1, y(t)) = 1;  # 1x3
  
  #step 2 (backpropagation)
  d3 = a3 - yv';  #3x1
#  a3
#  yv'  
#  d3
#  pause;
  
  #step 3 (delta: hidden layer)
  d2 = (Theta2' * d3);   #size: (6x3 * 3x1 = 6x1) = 6x1
  d2 = d2(2:end) .* sigmoidGradient(z2); # 5x1 .* 5x1 = 5x1
  #step 4: accumulate the gradient

#  fprintf("DD2 before\n");
#  DD2
  DD2 = DD2 + d3 * a2'; #3x1 * 1x6 = 3x6
#  fprintf("D2 = D2 + d3 * a2'\n");
#  DD2
#  pause;

#  fprintf("DD1 before\n");
#  DD1
  DD1 = DD1 + d2 * a1'; # 5x1 * 1x4 = 5x4
#  fprintf("DD1 = DD1 + d2 * a1'\n");
#  DD1
#  pause;
endfor

Theta1_grad = DD1/m ;
Theta2_grad = DD2/m ;

#recularization
Theta1_grad(:, 2:end) = DD1(:, 2:end)/m + lambda/m*Theta1(:, 2:end);
Theta2_grad(:, 2:end) = DD2(:, 2:end)/m + lambda/m*Theta2(:, 2:end);

#fprintf("Exit nnCostFunction\n");


% -------------------------------------------------------------

% =========================================================================

% Unroll gradients
grad = [Theta1_grad(:) ; Theta2_grad(:)];


end
