## Copyright (C) 2022 t.bajraszewski
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <https://www.gnu.org/licenses/>.

## -*- texinfo -*-
## @deftypefn {} {@var{retval} =} split_data (@var{data}, @var{share})
##  data: matrix m x (n+1) comprising m samples of n features + 1 labels
##       [x11, x12, ..., x1n, y1; x21, x22, ..., x2n, y2; ... ; xm1, xm2, ..., xmn, ym;]
##       labels y are indexed from 0.
##  share: [trainFraction, testFraction, xvalidFraction]
## @seealso{}
## @end deftypefn

## Author: t.bajraszewski <t.bajraszewski@S1473>
## Created: 2022-02-18

function [train, test, xvalid] = split_data (data, fraction, nLabels)

fprintf("Splitting data into train/test/valid:\r\n");
train = [];
test = [];
xvalid = [];

fprintf("Label\tQty\tNtrain\tNtest\tNvalid\r\n");

for i=1:nLabels

  label_idx = find(data(:, end) == (i-1));
  n = size(label_idx, 1);
  ind_shuffle = randperm(size(label_idx, 1));

  ia = round(n*fraction(1));
  train = [train; data(label_idx(ind_shuffle(1:ia)), :)];
  ib = ia + round(n * fraction(2));
  test = [test; data(label_idx(ind_shuffle(ia+1:ib)), :)];
  xvalid = [xvalid; data(label_idx(ind_shuffle(ib+1:end)), :)];

  fprintf("%d\t%d\t%d\t%d\t%d\r\n", i-1, n, ia, ib-ia-1, n-ib-1);

endfor
fprintf("Label\tQty\tNtrain\tNtest\tNvalid\n");

fprintf("Total\t%d\t%d\t%d\r\n", size(train,1), size(test,1), size(xvalid,1));

endfunction
